# Example no. 6
# Simply supported RC slab subjected to experimental temperatures
# Units are m, N, Pa, ton, and C
# Gravity in negative z 
# Example prepared by Mhd Anwar Orabi

# Problem description

# A rectangular 4.15 m x 3.15 m slab with 0.1 m total thickness, 25 mm clear concrete cover, and 200 mm2/m bottom reinforcement only
# Supported in z direction on all edges, and x and y translations and z rotation in one of its corners
# Load is a uniform load of 5.4 kN/m2 followed by thermal gradient over 1 hour as found in file temp.dat

# define model parameters

# general
set l 4.15
set w 3.15
set t 0.1
set cc 0.025
# 295 mm2/m translates to layer with thickness of 0.295e-3 m
set ro 0.000295
set UDL -5400
set halfT [expr 0.5001*$t]

# mesh 
set nx 10
set ny 10
set elemx [expr $l/$nx]
set elemy [expr $w/$ny]

# steel properties
set fy 568e6
set Es 2.06e11

# concrete properties
set fc 37e6
set ft [expr 0.1*$fc]
set v 0.2
set epscu 0.0025
set Ec [expr 1.5*$fc/$epscu]
set gt [expr $ft/$Ec*$ft*2]
set gc [expr $fc/$Ec*$fc*6]

####################################################################################

# Start up OpenSees model builder and create output folder

wipe
model BasicBuilder -ndm 3 -ndf 6
file mkdir output

# Load display engine
source DisplayPlane.tcl
source DisplayModel2D.tcl
source DisplayModel3D.tcl

####################################################################################

# Create nodes

# We want to number our nodes in this order:
# 11	22	33	44	55	66	77	88	99	110	121
# 10	21	32	43	54	65	76	87	98	109	120
#  9	20	31	42	53	64	75	86	97	108	119
#  8	19	30	41	52	63	74	85	96	107	118
#  7	18	29	40	51	62	73	84	95	106	117
#  6	17	28	39	50	61	72	83	94	105	116
#  5	16	27	38	49	60	71	82	93	104	115
#  4	15	26	37	48	59	70	81	92	103	114
#  3	14	25	36	47	58	69	80	91	102	113
#  2	13	24	35	46	57	68	79	90	101	112
#  1	12	23	34	45	56	67	78	89	100	111

# We can achieve this via the following nested loop
for {set i 0} {$i <= $nx} {incr i 1} {
	set x [expr $i*$elemx]
	for {set j 0} {$j <= $ny} {incr j 1} {
		set y [expr $j*$elemy]
		set nodeID [expr int($i*($ny+1) + $j + 1)]
		
	   #node nodeTag x  y  z
		node $nodeID $x $y 0
	}
}

# set any node tags you want to use later here:
set midNode [expr int($nx*0.5*($ny + 1) + 0.5*$ny + 1)]
puts "midnode = $midNode"

####################################################################################

# Create constraints

#fixX x u1  u2  u3 u11 u22 u33
fixX 0  0 	0 	1	0 	0 	0
fixX $l 0 	0 	1	0 	0 	0
fixY 0  0 	0 	1	0 	0 	0
fixY $w 0 	0 	1	0 	0 	0

#fix nodeTag u1  u2  u3 u11 u22 u33
fix  1   	 1 	 1 	 0 	 0 	 0 	 1
 
####################################################################################

# Create geometric transformation
# no geometric transformation requried for shell elements

####################################################################################

# Create material models

# Reinforcement
#uniaxialMaterial SteelECThermal matTag EC2NC yieldStrength YoungsModulus
 # uniaxialMaterial SteelECThermal 1 		EC2NC $fy 			$Es
   uniaxialMaterial Steel01Thermal 1  $fy $Es 0.015;
#nDMaterial PlateRebarThermal matTag uniaxialMatTag orientation(degrees)
 nDMaterial PlateRebarThermal 2 	 1 				0
 nDMaterial PlateRebarThermal 3 	 1 				90
 
# Concrete
#nDMaterial  CDPPlaneStressThermal matTag Ec  v  ft  fc  gt  gc
 nDMaterial  CDPPlaneStressThermal 4 	  $Ec $v $ft $fc $gt $gc
#nDMaterial   PlateFromPlaneStressThermal    matTag   	nDMatTag    forStability
 nDMaterial   PlateFromPlaneStressThermal    5   		4   		1e9
 
####################################################################################

# Create sections

#section LayeredShellThermal   secTag  numoflayers mat1Tag  mat1Thickness mat2Tag  mat2Thickness ... matnTag  matnThickness
section  LayeredShellThermal   1  	   13 		   5  		0.01 		  5  	   0.01  			 5 		  [expr 0.005 - $ro]			 2  $ro 	3 $ro	 5 [expr 0.01 - $ro]		 5 0.01		 5 0.01		 5 0.01		 5 0.01		 5 0.01		 5 0.01		 5 0.005;

####################################################################################

# Create elements

# We want to number our elements in this order:
# 10	20	30	40	50	60	70	80	90	100
#  9	19	29	39	49	59	69	79	89	99
#  8	18	28	38	48	58	68	78	88	98
#  7	17	27	37	47	57	67	77	87	97
#  6	16	26	36	46	56	66	76	86	96
#  5	15	25	35	45	55	65	75	85	95
#  4	14	24	34	44	54	64	74	84	94
#  3	13	23	33	43	53	63	73	83	93
#  2	12	22	32	42	52	62	72	82	92
#  1	11	21	31	41	51	61	71	81	91

# We can achieve this via the following nested loop
for {set i 1} {$i <= $nx} {incr i 1} {
	for {set j 1} {$j <= $ny} {incr j 1} {
	
		set elemID [expr int(($i - 1)*$ny + $j)]
		
		set node4 [expr int(($i - 1)*($ny + 1) + $j)]
		set node3 [expr $node4 + 1]
		set node2 [expr $node3 + $ny + 1]
		set node1 [expr $node2 - 1]
		
		#element ShellNLDKGQThermal	 $eleTag $iNode $jNode $kNode $lNode $secTag
		 element ShellNLDKGQThermal	 $elemID $node1 $node2 $node3 $node4 1
	}
}

####################################################################################

# Call graphical engine

set  xPixels 1200;	# height of graphical window in pixels
set  yPixels 800;	# height of graphical window in pixels
set  xLoc1 10;	# horizontal location of graphical window (0=upper left-most corner)
set  yLoc1 10;	# vertical location of graphical window (0=upper left-most corner)
set ViewScale 0.001;	# scaling factor for viewing deformed shape, it depends on the dimensions of the model
DisplayModel3D  DeformedShape $ViewScale $xLoc1 $yLoc1  $xPixels $yPixels

####################################################################################

# Create recorders

recorder Node -file output/midNodeU3.out -time -node $midNode -dof 3 disp
recorder Element -file output/ele45Tlayer1.out -time -ele 45   material 1 fiber 1 TempAndElong;
recorder Element -file output/ele45Tlayer12.out -time -ele 45   material 1 fiber 12 TempAndElong;

####################################################################################

# Create ambient loads
set f3 [expr $UDL*$elemx*$elemy]
pattern Plain 1 Linear {
	for {set i 1} {$i <= [expr ($nx + 1)*($ny + 1)]} {incr i 1} {
	   #load nodeTag	f1 	f2 	f3 	 f11  f22  f33
		load  $i 		0   0   $f3  0 	  0    0
	}
}

####################################################################################

# Analyse ambient loads

set ok 0
set stepSize 0.05
set currentTime 0.0
set tFinal 1.0

constraints Transformation    		
numberer RCM			
system SparseSYM		
test NormDispIncr 1e-4 100 1	
algorithm Newton				
integrator LoadControl $stepSize				
analysis Static

while {$currentTime < $tFinal && $ok == 0} {
	puts "Attempting analysis for time [expr $currentTime + $stepSize], with step size = $stepSize"
	set ok [analyze 1];
	if {$ok == 0} {
		set currentTime [getTime]
		puts "Analysis step successful. Current time is $currentTime"
	} else {
		puts "Analysis failed at time = $currentTime"
		return
	}
}
loadConst -time 0.0

####################################################################################

# Create thermal loads

pattern Plain 2 Linear {
	   #eleLoad -range firstNode lastNode 		-type -shellThermal -source "fileName" lowestLayer  $highestLayer
		eleLoad -range 1 		 [expr $nx*$ny] -type -shellThermal -source "temp.dat" -$halfT 		$halfT
}

####################################################################################

# Analyse Thermal loads

set ok 0
set stepSize 15
set currentTime 0.0
set tFinal 10800

constraints Plain    		
numberer Plain			
system BandGeneral		
test NormDispIncr 1e-3 500 1	
algorithm Newton				
integrator LoadControl $stepSize				
analysis Static

while {$currentTime < $tFinal && $ok == 0} {
	puts "Attempting analysis for time [expr $currentTime + $stepSize], with step size = $stepSize"
	set ok [analyze 1];
	if {$ok == 0} {
		set currentTime [getTime]
		puts "Analysis step successful. Current time is $currentTime"
	} else {
		puts "Analysis failed at time = $currentTime"
		return
	}
}